from typing import Any

import numpy as np
import openai
from numpy import ndarray, dtype

from .shared import load_openai_api_key


class OpenAIConnector:
    def __init__(self, api_key):
        self.client = openai.Client(api_key=api_key)

    def embed_documents(self, documents: list[str], embedding_model="text-embedding-3-small") -> list[
        ndarray[Any, dtype[Any]]]:
        embeddings = self.client.embeddings.create(input=documents, model=embedding_model)
        return [np.array(d.embedding) for d in embeddings.data]

    def chat(self, prompt: str, model: str = 'gpt-3.5-turbo', temperature: float = 0.0, max_tokens: int = 100) -> str:
        completion = self.client.chat.completions.create(
            model=model,
            messages=[
                {"role": "user", "content": prompt},
            ],
            max_tokens=max_tokens,
            temperature=temperature
        )
        return completion.choices[0].message.content


if __name__ == "__main__":
    # Load API key
    api_key = load_openai_api_key("../openai_api_key.json")
    connector = OpenAIConnector(api_key)

    # Embed documents
    documents = ["Hello, world!", "How are you?"]
    embedding_model = "text-embedding-3-small"
    embeddings = connector.embed_documents(documents, embedding_model)
    print(embeddings, embeddings.shape)

    # Chat with GPT-3.5 Turbo
    prompt = "What is the meaning of life?"
    response = connector.chat(prompt)
    print(response)



from qdrant_client import QdrantClient
from qdrant_client.http.models import PointStruct
from qdrant_client.http.models import VectorParams, Distance


class VectorOps:
    def __init__(self):
        self.client = QdrantClient(":memory:")

    def make_collection(self, documents_with_embeddings, collection_name="letters"):
        documents = documents_with_embeddings['text']
        embeddings = documents_with_embeddings['embedding']

        points = [PointStruct(id=idx, vector=e, payload={'text': d})
                  for idx, (d, e) in enumerate(zip(documents, embeddings))]

        print(f"Creating collection {collection_name} with {len(points)} points of size {embeddings[0].shape[0]}")
        self.client.create_collection(collection_name=collection_name,
                                      vectors_config=VectorParams(size=embeddings[0].shape[0],
                                                                  distance=Distance.COSINE))

        self.client.upsert(collection_name, points)

    def find_similar(self, embedding, collection_name="letters", top_k=5, min_similarity=0.1):
        return self.client.search(collection_name=collection_name, query_vector=embedding,
                                  limit=top_k, score_threshold=min_similarity)



import io
from pathlib import Path

import pandas as pd


class DocumentLoader:

    def _read_txt(self, path: Path) -> str:
        with io.open(path, 'r', encoding='utf-8') as f:
            return f.read()

    def bulk_load_documents(self, path: Path, ext='.txt') -> pd.DataFrame:
        ext = '.' + ext.lower().lstrip('.')
        documents = []
        documents_ids = []
        for file in path.glob(f'*{ext}'):
            with io.open(file, 'r', encoding='utf-8') as f:
                if ext == '.txt':
                    documents.append(self._read_txt(file))
                else:
                    raise ValueError(f'Unsupported file format: {ext}')
                documents_ids.append(file.stem)
        return pd.DataFrame({'id': documents_ids, 'text': documents})

    def load_document(self, path: Path, ext='.txt') -> str:
        if ext == '.txt':
            return self._read_txt(path)
        else:
            raise ValueError(f'Unsupported file format: {ext}')
